package gui;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

/**
 *  The ButtonColumn class provides a renderer and an editor that looks like a
 *  JButton. The renderer and editor will then be used for a specified column
 *  in the table. The TableModel will contain the String to be displayed on
 *  the button.
 *
 *  The button can be invoked by a mouse click or by pressing the space bar
 *  when the cell has focus. Optionaly a mnemonic can be set to invoke the
 *  button. When the button is invoked the provided Action is invoked. The
 *  source of the Action will be the table. The action command will contain
 *  the model row number of the button that was clicked.
 *
 */
public class ButtonColumn extends AbstractCellEditor
	implements TableCellRenderer, TableCellEditor, ActionListener, MouseListener
{       
        /*
        //Double Click Start

        static long oldctm = System.currentTimeMillis();  
        final int maxDoubleTime = 1000;  
        ActionListener waitOneClickCount  = new ActionListener () {  
              public void actionPerformed (ActionEvent evt) {  
                System.out.println (String.valueOf (System.currentTimeMillis() - oldctm) +  
                          " oneShot timeout - single click action here");  
              }  
        };  
        Timer oneShot = new Timer (maxDoubleTime, waitOneClickCount);  

        //Double Click end
        */
        
        
        
        
	private JTable table;
	private Action action;
	private int mnemonic;
	private Border originalBorder;
	private Border focusBorder;

	private MeuJButton renderButton;
	private MeuJButton editButton;
	private String text;
	private boolean isButtonColumnEditor;


        private ChatWindow chatScreen;
	/**
	 *  Create the ButtonColumn to be used as a renderer and editor. The
	 *  renderer and editor will automatically be installed on the TableColumn
	 *  of the specified column.
	 *
	 *  @param table the table containing the button renderer/editor
	 *  @param action the Action to be invoked when the button is invoked
	 *  @param column the column to which the button renderer/editor is added
	 */
	public ButtonColumn(JTable table, Action action, int column)
	{
		this.table = table;
		this.action = action;
		renderButton = new MeuJButton();
		editButton = new MeuJButton();
                
                renderButton.setFocusPainted(false);
                renderButton.setMargin(new Insets(0, 0, 0, 0));
                renderButton.setContentAreaFilled(false);
                renderButton.setBorderPainted(false);
                renderButton.setOpaque(false);
                renderButton.setFont(new java.awt.Font("Tahoma", 0, 14));
                renderButton.setHorizontalAlignment(SwingConstants.LEFT);
                
                editButton.setFocusPainted(false);
                editButton.setMargin(new Insets(0, 0, 0, 0));
                editButton.setFont(new java.awt.Font("Tahoma", 0, 14));
                editButton.setHorizontalAlignment(SwingConstants.LEFT);
                
                this.table.setRowHeight(30);

		editButton.setFocusPainted( false );
		editButton.addActionListener( this );
		originalBorder = editButton.getBorder();
		setFocusBorder( new LineBorder(Color.BLUE) );
                

		TableColumnModel columnModel = table.getColumnModel();
		columnModel.getColumn(column).setCellRenderer( this );
		columnModel.getColumn(column).setCellEditor( this );
		table.addMouseListener( this );
	}


	/**
	 *  Get foreground color of the button when the cell has focus
	 *
	 *  @return the foreground color
	 */
	public Border getFocusBorder()
	{
		return focusBorder;
	}

	/**
	 *  The foreground color of the button when the cell has focus
	 *
	 *  @param focusBorder the foreground color
	 */
	public void setFocusBorder(Border focusBorder)
	{
		this.focusBorder = focusBorder;
		editButton.setBorder( focusBorder );
	}

	public int getMnemonic()
	{
		return mnemonic;
	}

	/**
	 *  The mnemonic to activate the button when the cell has focus
	 *
	 *  @param mnemonic the mnemonic
	 */
	public void setMnemonic(int mnemonic)
	{
		this.mnemonic = mnemonic;
		renderButton.setMnemonic(mnemonic);
		editButton.setMnemonic(mnemonic);
	}

	@Override
	public Component getTableCellEditorComponent(
		JTable table, Object value, boolean isSelected, int row, int column)
	{
		text = (value == null) ? "" : value.toString();
		editButton.setText( text );
		return editButton;
	}

	@Override
	public Object getCellEditorValue()
	{
		return text;
	}

//
//  Implement TableCellRenderer interface
//
	public Component getTableCellRendererComponent(
		JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column)
	{
		if (isSelected)
		{
			renderButton.setForeground(table.getSelectionForeground());
	 		renderButton.setBackground(table.getSelectionBackground());
		}
		else
		{
			renderButton.setForeground(table.getForeground());
			renderButton.setBackground(UIManager.getColor("Button.background"));
		}

		if (hasFocus)
		{
			renderButton.setBorder( focusBorder );
		}
		else
		{
			renderButton.setBorder( originalBorder );
		}

		renderButton.setText( (value == null) ? "" : value.toString() );
		return renderButton;
	}

//
//  Implement ActionListener interface
//
	/*
	 *	The button has been pressed. Stop editing and invoke the custom Action
	 */
	public void actionPerformed(ActionEvent e)
	{
		int row = table.convertRowIndexToModel( table.getEditingRow() );
		fireEditingStopped();

                //action here
                
		ActionEvent event = new ActionEvent(
			table,
			ActionEvent.ACTION_PERFORMED,
			"" + row);
		action.actionPerformed(event);
	}

//
//  Implement MouseListener interface
//
	/*
	 *  When the mouse is pressed the editor is invoked. If you then then drag
	 *  the mouse to another cell before releasing it, the editor is still
	 *  active. Make sure editing is stopped when the mouse is released.
	 */
    public void mousePressed(MouseEvent e)
    {
            //Double Click Start
            /*
            long ctm = System.currentTimeMillis();  
            long dif_ctm = ctm - oldctm;  
            int clickCount = e.getClickCount ();  
            String ps = e.paramString ();  
            System.out.println ("ctm=" + ctm +", dif=" + dif_ctm +  
                        ", clickCount=" + clickCount);  
            if ((dif_ctm < maxDoubleTime) || (clickCount > 1)) {  
                System.out.println ("Double click detected");  
                //Old Code start
                

                
                //Old Code end
                
                oneShot.stop ();  
            } else {  
                 System.out.println ("Single click detected - oneShot started");  
                 oneShot.setRepeats (false);  
                 oneShot.start ();  
            }  
            oldctm = ctm;  
            */
            //Double Click End
        
                        if (table.isEditing()
                    &&  table.getCellEditor() == this)
			isButtonColumnEditor = true;
        
           
   }

    public void mouseReleased(MouseEvent e)
    {
    	if (isButtonColumnEditor
    	&&  table.isEditing())
    		table.getCellEditor().stopCellEditing();

		isButtonColumnEditor = false;
    }

    public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
}
